import React from "react"
import { useState } from "react"
import { useContextMenu } from "react-contexify"
import { RouteComponentProps } from "react-router-dom"
import downloadFile from "../functions/downloadFile"
import genDownloadLink from "../functions/genDownloadLink"
import uploadFile from "../functions/uploadFile"
import { useCopyMutation, useCreateDirMutation, useDeleteDirMutation, useDeleteFileMutation, useMoveDirMutation, useMoveMutation, useOpenDirQuery } from "../generated/graphql"
import Breadcrum from "./Breadcrum"
import CreateDirButton from "./CreateDirButton"
import GlobalDragAndDrop from "./GlobalDragAndDrop"
import FileBrowserContextMenu, { Action, CONTEXT_MENU_DIR, CONTEXT_MENU_FILE } from "./FileBrowserContextMenu"
import FileOpen from "./FileOpen"
import FileUploadButton from "./FileUploadButton"
import { ReactComponent as Spinner } from "./../assets/spinner.svg"
import FileBrowserList from "./FileBrowserList"
import MoreMenu from "./MoreMenu"
import ObjID from "../types/ObjID"
import cache from "../Cache"

const FileBrowser: React.FC<RouteComponentProps> = (props) => {
	const path = ObjID.fromURI(props.location.pathname)
	const [openFileId, setOpenFileId] = useState<ObjID>()
	const [showFile, setShowFile] = useState(false)

	const [srcID,setSrcID] = useState<ObjID | null>()
	const [pasteAction,setPasteAction] = useState<Action>()
	const [editID,setEditID] = useState<ObjID>()
	const [editEnable,setEditEnable] = useState(false)

	const [deleteMutation] = useDeleteFileMutation()
	const [copyMutation] = useCopyMutation()
	const [moveMutation] = useMoveMutation()
	const [createDirMutation] = useCreateDirMutation()
	const [deleteDirMutation] = useDeleteDirMutation()
	const [moveDirMutation] = useMoveDirMutation()

	const { show: showFileContext } = useContextMenu({
		id: CONTEXT_MENU_FILE,
	})

	const { show: showDirContext } = useContextMenu({
		id: CONTEXT_MENU_DIR,
	})

	const { data, loading, refetch: refetchDir } = useOpenDirQuery({
		variables:{
			path: path
		}
	})

	async function handleDrop(files:FileList) {
		const wait: Promise<boolean>[] = []
		for (let i = 0; i < files.length; i++) {
			const file = files[i]
			wait.push(uploadFile(file, new ObjID(path.bucket,path.key + file.name)))
		}

		await Promise.all(wait)
		refetchDir()
	}

	async function onContextSelect(action:Action, id: ObjID) {
		switch (action) {
		case Action.FileDelete:
			await deleteMutation({variables:{id}})
			refetchDir()
			break
		case Action.FileCopy:
		case Action.FileMove:
		case Action.DirMove:
			setSrcID(id)
			setPasteAction(action)
			break
		case Action.FilePaste:
			if (pasteAction === Action.FileCopy && srcID){
				await copyMutation({variables:{src:srcID,dest:path}})
				refetchDir()
			}

			if (pasteAction === Action.FileMove && srcID){
				await moveMutation({variables:{src:srcID,dest:path}})
				refetchDir()
				setSrcID(null)
			}

			if (pasteAction === Action.DirMove && srcID){
				await moveDirMutation({variables:{src:srcID,dest:path}})
				refetchDir()

				cache.evict({
					id: `Directory:${srcID.toString()}` // TODO: check for a better way to generate cache ids
				})
				
				setSrcID(null)
			}

			break
		case Action.DirDelete:
			await deleteDirMutation({variables:{path:id}})
			refetchDir()
			break
		case Action.FileDownload:
			downloadFile(genDownloadLink(id))
			break
		case Action.FileRename:
			setEditID(id)
			setEditEnable(true)
			break
		default:
			break
		}
	}

	return (
		<div>
			<FileBrowserContextMenu
				onSelect={onContextSelect}
				pasteActive={!!srcID}
			/>
			<GlobalDragAndDrop
				handleDrop={async (files)=>{
					await handleDrop(files)
				}}/>
			<div className="flex justify-between h-12">
				<Breadcrum path={path} onDirClick={(newPath)=>{
					props.history.push(newPath.toURI())
				}}/>
				<div className="ml-auto">
					<CreateDirButton 
						onPressed={async (dirName)=>{
							const dirID = new ObjID(path.bucket,path.key + dirName + "/")
							await createDirMutation({variables:{path: dirID}})
							refetchDir()
						}}
					/>
				</div>
				<div>
					<FileUploadButton 
						onUpload={(files)=>handleDrop(files)}
					/>
				</div>
				<div>
					<MoreMenu />
				</div>

			</div>
			<div>
				<FileBrowserList 
					directorys={data?.directories || []}
					files={data?.files || []}

					onDirClick={(e,path)=>{
						props.history.push(path.toURI())
					}}

					onDirContext={(e,path)=>{
						e.preventDefault()
						showDirContext(e,{props:{id:path}})
					}}

					onFileClick={(e,id)=>{
						setOpenFileId(id)
						setShowFile(true)
					}}

					onFileContext={(e,id)=>{
						e.preventDefault()
						showFileContext(e,{props:{id}})
					}}

					editId={editID}
					editEnable={editEnable}

					onRenameDone={async (id,changed,newName)=>{
						setEditEnable(false)
						if (changed){
							// TODO: maybe change the name on client so it seems more smooth rather then haveing it refetch
							// TODO: input check & error handling
							await moveMutation({variables:{
								src:id,
								dest: id.rename(newName)
							}})
							refetchDir()
						}
					}}
				/>
				{loading && 
					<div className="flex justify-center mt-4">
						<Spinner className="animate-spin h-6 w-6 dark:text-white"  />
					</div>
				}
			</div>
			<FileOpen id={openFileId} show={showFile} onCloseClick={()=>{
				setShowFile(false)
			}} />
		</div>
	)
}

export default FileBrowser
